function [Props,AsgnVals] = pnames(lincon)
%PNAMES  All public properties and their assignable values
%
%   [PROPS,ASGNVALS] = PNAMES(lincon)  returns the list PROPS of
%   public properties of the object lincon (a cell vector), as well as 
%   the assignable values ASGNVALS for these properties (a cell vector
%   of strings).  PROPS contains the true case-sensitive property names.
%
%   See also  GET, SET.

%   (C) 2003 by A. Bemporad


% LINCON public properties

% Remember to be consistent with LINCON.M in case of empty @LINCON objects
Props = {'Q','C','G','W','S','model','nx','nu','ny',...
        'type','ts','isconstr','soft','nvar','nq','npar','QPsolver','I1','Qinv',...
        'Observer'};
   
% Also return assignable values if needed
if nargout>1,
   AsgnVals = {
         'nvar-by-nvar array (Hessian matrix)';...
         'nvar-by-npar array (linear cost matrix)';...
         'nq-by-nvar array (lhs constraint matrix)';...
         'nq-by-1 array (rhs constraint vector)';...
         'nq-by-npar array (rhs constraint matrix)';...
         'LTI object (linear discrete-time model)';...
         'scalar (number of states)';...
         'scalar (number of inputs)';...
         'scalar (number of outputs)';...
         'char array (controller type)';...
         'scalar (sampling time)';...
         'scalar (controller constrained/unconstrained)';...
         'scalar (soft constraints/hard constraints)';...
         'scalar (number of optimization variables)';...
         'scalar (number of linear inequality constraints)';...
         'scalar (number of parameters (states and references))';...
         'char array (QP solver)';...
         'nu-by-nvar array (extracts u(0) from optimal sequence)';...
         'nvar-by-nvar array (inverse of Hessian matrix)';...
         'char array or structure (observer information)';...
};
end

% end lincon/pnames.m
