function [Props,AsgnVals] = pnames(MLD)
%PNAMES  All public properties and their assignable values
%
%   [PROPS,ASGNVALS] = PNAMES(MLD)  returns the list PROPS of
%   public properties of the object MLD (a cell vector), as well as 
%   the assignable values ASGNVALS for these properties (a cell vector
%   of strings).  PROPS contains the true case-sensitive property names.
%
%   See also  GET, SET.

%   (C) 2003 by A. Bemporad


% MLD public properties

Props = {'ts';'name';'simname';'A';'B1';'B2';'B3';'C';'D1';'D2';'D3';...
        'E1';'E2';'E3';'E4';'E5';'rowinfo';'symtable';...
        'ne';'nx';'nu';'ny';'nd';'nz';'nxr';'nxb';'nur';'nub';'nyr';'nyb';'milpsolver';...
        'params';'hysmodel'};

% Also return assignable values if needed
if nargout>1,
    AsgnVals = {
        'scalar (sampling time)';...
            'string (system name, as defined in the HYSDEL source)';...
            'open loop simulator name';...
            'nx-by-nx array';...
            'nx-by-nu array';...
            'nx-by-nd array';...
            'nx-by-nz array';...
            'ny-by-nx array';...
            'ny-by-nu array';...
            'ny-by-nd array';...
            'ny-by-nz array';...
            'ne-by-nx array';...
            'ne-by-nu array';...
            'ne-by-nd array';...
            'ne-by-nz array';...
            'ne-by-1  array';...
            'struct';...
            'struct';...
            'scalar (number of mixed-integer linear inequalities)';...
            'scalar (total number of states)';...
            'scalar (total number of inputs)';...
            'scalar (total number of outputs)';...
            'scalar (number of binary auxiliary vars)';...
            'scalar (number of continuous auxiliary vars)';...
            'scalar (number of continuous states)';...
            'scalar (number of binary states)';...
            'scalar (number of continuous inputs)';...
            'scalar (number of binary inputs)';...
            'scalar (number of continuous outputs)';...
            'scalar (number of binary outputs)';...
            'string (MILP solver)';...
            'structure (HYSDEL symbolic parameters)';...
            'string (HYSDEL filename)';...
        };
end

% end MLD/pnames.m
