function [Props,AsgnVals] = pnames(PWA)
%PNAMES  All public properties and their assignable values
%
%   [PROPS,ASGNVALS] = PNAMES(PWA)  returns the list PROPS of
%   public properties of the object PWA (a cell vector), as well as 
%   the assignable values ASGNVALS for these properties (a cell vector
%   of strings).  PROPS contains the true case-sensitive property names.
%
%   See also  GET, SET.

%   (C) 2003 by A. Bemporad

% PWA public properties

Props = {'nr';'A';'B';'f';'Hx';'Hu';'K';'LA';'LB';'Lf';...
      'C';'D';'g';'LC';'LD';'Lg';'logic';...
      'nx';'nxr';'nxb';'nu';'nur';'nub';'ny';'nyr';'nyb';'name';'ts';'mld';
  'lpsolver';'milpsolver';'hysmodel'};
   
% Also return assignable values if needed
if nargout>1,
   AsgnVals = {'scalar';...
         'cell array of nxr-by-nxr arrays';...
         'cell array of nxr-by-nur arrays';...
         'cell array of nxr-by-1 arrays';...
         'cell array of nqi-by-nxr arrays';...
         'cell array of nqi-by-nur arrays';...
         'cell array of nqi-by-1 arrays';...
         'cell array of nxb-by-nxr arrays';...
         'cell array of nxb-by-nur arrays';...
         'cell array of nxb-by-1 arrays';...
         'cell array of nyr-by-nxr arrays';...
         'cell array of nyr-by-nur arrays';...
         'cell array of nyr-by-1 arrays';...
         'cell array of nyb-by-nxr arrays';...
         'cell array of nyb-by-nur arrays';...
         'cell array of nyb-by-1 arrays';...
         'cell array of combinations of [xb;ub;delta] variables';...
         'scalar (total number of states)';...
         'scalar (number of continuous states)';...
         'scalar (number of binary states)';...
         'scalar (total number of inputs)';...
         'scalar (number of continuous inputs)';...
         'scalar (number of binary inputs)';...
         'scalar (total number of outputs)';...
         'scalar (number of continuous outputs)';...
         'scalar (number of binary outputs)';...
         'string (HYSDEL list name)';...
         'scalar (sampling time)';...
         'string (MLD variable name)';...
         'string (predefined LP solver)';...
         'string (predefined MILP solver)';...
         'string (HYSDEL filename)';...
};
end

% end PWA/pnames.m
