% CDDMEX: Matlab MEX file v0.02 obtained by linking CDDLIB v0.92a
% (code obtained by Fabio D. Torrisi)
%
% Syntax: outstruct=cddmex(action,instruct)
%
% action:  'hull'             Compute convex hull 
%          'extreme'          Vertex/ray enumeration
%          'reduce_h'         Find minimal H representation
%          'copy_v'
%          'copy_h'
%          'v_hull_extreme'
%          'adj_extreme'
%          'solve_lp'         Solve a Linear Program
%          'version'
%
% Further info: see CDD.C
%
% Example: Find the extreme points/rays of P={x: A1*x=B1, A2*x<=B1}
%
%    A1=[1 1 1];B1=[1];
%    A2=[eye(3);-eye(3)];B2=[1;1;1;2;2;2];
%    H=struct('A',[A1;A2],'B',[B1;B2],'lin',(1:size(B1,1))');
%    % H.lin=indices of equality constraints
%    V=cddmex('extreme',H);
%    % The rows of V.V are the extreme points of P
%
% Example: Find the convex hull of v1,v2,v3
%
%    V=struct('V',[v1';v2';v3']);
%    H=cddmex('hull',V);
%
% Example: Find minimal H representation
%
%    A1=[-1 0; 0 -1; 1 0; 0 1; 1 1];B1=[0;0;1;1;1];
%    H1=struct('A',A1,'B',B1);
%    [Hred]=cddmex('reduce_h',H1);
%    %  Hred.A * x <= Hred.B  is a minimal H representation
%    [Hred,ind_redrows]=cddmex('reduce_h',H1);
%    %  ind_redrows are the indices of redundant rows
%    % Note: if H1 is empty polyhedron, then Hred
%    % containes irreducibly incosistent set (IIS)
%
% Example: Solve a Linear Program
% 		Syntax
%          OUT=cdd('solve_lp',IN)
% 		
% 		solves problem
%          min	IN.obj*x
%           x
%          s.t.	IN.A*x <= IN.B
%          	IN.A(IN.lin,:) x == IN.B(IN.lin)
% 		
% 		where
%           IN  -	input structure
%          	IN.A	- constraints matrix
%          	IN.B	- constraints matrix
%          	IN.obj	- objective function
%          	IN.lin	- indices of equality constraints
%          	
%           OUT -	structure with solution (similiar to lpsolve)
%          	OUT.xopt	- primal solution
%          	OUT.lambda	- dual solution
%          	OUT.how		- Fukuda's code for LP solution status
%          			  0 = dd_LPSundecided,
%          			  1 = dd_Optimal 
%          			  2 = dd_Incosistent
%          			  3 = dd_DualIncosistent
%          			  4 = dd_StrucIncosistent
%          			  5 = dd_StrucDualIncosistent
%          			  6 = dd_Unbounded
%          			  7 = dd_DualUnbounded
%          	OUT.objlp	- optimal value
%          	
% 		Example
% 			objective=[-1 1];
% 			A1=[1 1; -1 0; 0 -1];
% 			B1=[1; 0; 0];
%        IN=struct('obj',objective,'A',A1,'B',B1);
% 			
% 			OUT = cdd('solve_lp',IN);
% 			
% 			OUT =
% 			    xopt: [1 0]
% 			    lambda: [-1 0 -2]
% 			    how: 1
% 			    objlp: -1
%
%
% Note: the projection function is only available in CDD.M. A projection
% can be obtained here by (1) vertex enumeration, (2) projection of vertices,
% (3) hull of projected vertices. Example:
%
%    H=struct('A',A1,'B',B1);
%    V=cddmex('extreme',H);
%    Vproj.V=V.V(:,1:2); % Project over the first two coordinates
%    Hproj=cddmex('hull',Vproj); % Get the projection
%    polyplot(Hproj.A,Hproj.B);
%    xlabel('x_1');
%    ylabel('x_2');

